@@ -1,4 +1,4 @@
-## Copyright (C) 2007, 2008 David Bateman
+## Copyright (C) 2007-2012 David Bateman
 ##
 ## This file is part of Octave.
 ##
@@ -17,14 +17,22 @@
 ## <http://www.gnu.org/licenses/>.
 
 function geometryimages (nm, typ)
-  bury_output ();
+  graphics_toolkit ("gnuplot");
+  set_print_size ();
+  hide_output ();
   if (strcmp (typ, "png"))
     set (0, "defaulttextfontname", "*");
   endif
+  if (strcmp (typ, "eps"))
+    d_typ = "-depsc2";
+  else
+    d_typ = cstrcat ("-d", typ);
+  endif
+
   if (isempty (findstr (octave_config_info ("DEFS"), "HAVE_QHULL"))
       && (strcmp (nm, "voronoi") || strcmp (nm, "griddata")
-	  || strcmp (nm, "convhull") || strcmp (nm, "delaunay")
-	  || strcmp (nm, "triplot")))
+          || strcmp (nm, "convhull") || strcmp (nm, "delaunay")
+          || strcmp (nm, "triplot")))
     sombreroimage (nm, typ);
   elseif (strcmp (typ, "txt"))
     image_as_txt (nm);
@@ -44,14 +52,14 @@ function geometryimages (nm, typ)
     plot (xc, yc, "g-", "LineWidth", 3);
     axis([0, 1, 0, 1]);
     legend ("Delaunay Triangulation", "Voronoi Diagram");
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
+    print (cstrcat (nm, ".", typ), d_typ)    
   elseif (strcmp (nm, "triplot"))
     rand ("state", 2)
     x = rand (20, 1);
     y = rand (20, 1);
     tri = delaunay (x, y);
     triplot (tri, x, y);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
+    print (cstrcat (nm, ".", typ), d_typ)    
   elseif (strcmp (nm, "griddata"))
     rand("state",1);
     x=2*rand(1000,1)-1;
@@ -59,14 +67,14 @@ function geometryimages (nm, typ)
     z=sin(2*(x.^2+y.^2));
     [xx,yy]=meshgrid(linspace(-1,1,32));
     griddata(x,y,z,xx,yy);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
+    print (cstrcat (nm, ".", typ), d_typ)    
   elseif (strcmp (nm, "convhull"))
     x = -3:0.05:3;
     y = abs (sin (x));
     k = convhull (x, y);
     plot (x(k),y(k),'r-',x,y,'b+');
     axis ([-3.05, 3.05, -0.05, 1.05]);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ)) 
+    print (cstrcat (nm, ".", typ), d_typ) 
   elseif (strcmp (nm, "delaunay"))
     rand ("state", 1);
     x = rand (1, 10);
@@ -76,7 +84,7 @@ function geometryimages (nm, typ)
     Y = [ y(T(:,1)); y(T(:,2)); y(T(:,3)); y(T(:,1)) ];
     axis ([0, 1, 0, 1]);
     plot(X, Y, "b", x, y, "r*");
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ)) 
+    print (cstrcat (nm, ".", typ), d_typ) 
   elseif (strcmp (nm, "inpolygon"))
     randn ("state", 2);
     x = randn (100, 1);
@@ -86,11 +94,11 @@ function geometryimages (nm, typ)
     in = inpolygon (x, y, vx, vy);
     plot(vx, vy, x(in), y(in), "r+", x(!in), y(!in), "bo");
     axis ([-2, 2, -2, 2]);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ)) 
+    print (cstrcat (nm, ".", typ), d_typ) 
   else
     error ("unrecognized plot requested");
   endif
-  bury_output ();
+  hide_output ();
 endfunction
 
 function [r, c] = tri2circ (tri, xx, yy)
@@ -104,68 +112,19 @@ function [r, c] = tri2circ (tri, xx, yy)
   r = sqrt ((xc - x(1)).^2 + (yc - y(1)).^2);
 endfunction
 
-## Use this function before plotting commands and after every call to
-## print since print() resets output to stdout (unfortunately, gnpulot
-## can't pop output as it can the terminal type).
-function bury_output ()
-  f = figure (1);
-  set (f, "visible", "off");
-endfunction
-function geometryimages (nm, typ)
-  bury_output ();
-  if (strcmp (nm, "voronoi"))
-    rand("state",9);
-    x = rand(10,1);
-    y = rand(10,1);
-    tri = delaunay (x, y);
-    [vx, vy] = voronoi (x, y, tri);
-    triplot (tri, x, y, "b");
-    hold on;
-    plot (vx, vy, "r");
-    [r, c] = tri2circ (tri(end,:), x, y);
-    pc = [-1:0.01:1];
-    xc = r * sin(pi*pc) + c(1);
-    yc = r * cos(pi*pc) + c(2);
-    plot (xc, yc, "g-", "LineWidth", 3);
-    axis([0, 1, 0, 1]);
-    legend ("Delaunay Triangulation", "Voronoi Diagram");
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
-  elseif (strcmp (nm, "triplot"))
-    rand ("state", 2)
-    x = rand (20, 1);
-    y = rand (20, 1);
-    tri = delaunay (x, y);
-    triplot (tri, x, y);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
-  elseif (strcmp (nm, "griddata"))
-    rand("state",1);
-    x=2*rand(1000,1)-1;
-    y=2*rand(size(x))-1;
-    z=sin(2*(x.^2+y.^2));
-    [xx,yy]=meshgrid(linspace(-1,1,32));
-    griddata(x,y,z,xx,yy);
-    print (cstrcat (nm, ".", typ), cstrcat ("-d", typ))    
-  else
-    error ("unrecognized plot requested");
-  endif
-  bury_output ();
-endfunction
-
-function [r, c] = tri2circ (tri, xx, yy)
-  x = xx(tri);
-  y = yy(tri);
-  m = (y(1:end-1) - y(2:end)) ./ (x(1:end-1) - x(2:end));
-  xc = (prod(m) .* (y(1) - y(end)) + m(end)*(x(1)+x(2)) - m(1)*(x(2)+x(3))) ...
-        ./ (2 * (m(end) - m(1))); 
-  yc = - (xc - (x(2) + x(3))./2) ./ m(end) + (y(2) + y(3)) / 2;
-  c = [xc, yc];
-  r = sqrt ((xc - x(1)).^2 + (yc - y(1)).^2);
+function set_print_size ()
+  image_size = [5.0, 3.5]; # in inches, 16:9 format
+  border = 0;              # For postscript use 50/72
+  set (0, "defaultfigurepapertype", "<custom>");
+  set (0, "defaultfigurepaperorientation", "landscape");
+  set (0, "defaultfigurepapersize", image_size + 2*border);
+  set (0, "defaultfigurepaperposition", [border, border, image_size]);
 endfunction
 
 ## Use this function before plotting commands and after every call to
 ## print since print() resets output to stdout (unfortunately, gnpulot
 ## can't pop output as it can the terminal type).
-function bury_output ()
+function hide_output ()
   f = figure (1);
   set (f, "visible", "off");
 endfunction
@@ -181,7 +140,12 @@ function sombreroimage (nm, typ)
     return;
   else ## if (!strcmp (typ, "txt"))
 
-    bury_output ();
+    hide_output ();
+    if (strcmp (typ, "eps"))
+      d_typ = "-depsc2";
+    else
+      d_typ = cstrcat ("-d", typ);
+    endif
 
     x = y = linspace (-8, 8, 41)';
     [xx, yy] = meshgrid (x, y);
@@ -191,8 +155,8 @@ function sombreroimage (nm, typ)
       mesh (x, y, z);
       title ("Sorry, graphics not available because octave was\\ncompiled without the QHULL library.");
     unwind_protect_cleanup
-      print (cstrcat (nm, ".", typ), cstrcat ("-d", typ));
-      bury_output ();
+      print (cstrcat (nm, ".", typ), d_typ);
+      hide_output ();
     end_unwind_protect
   endif
 endfunction
