@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1996-2011 John W. Eaton
+Copyright (C) 1996-2012 John W. Eaton
 
 This file is part of Octave.
 
@@ -42,6 +42,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-env.h"
 #include "oct-mutex.h"
 #include "oct-time.h"
+#include "singleton-cleanup.h"
 
 command_editor *command_editor::instance = 0;
 
@@ -765,7 +766,7 @@ std::string
 default_command_editor::do_readline (const std::string& prompt, bool& eof)
 {
   gnulib::fputs (prompt.c_str (), output_stream);
-  fflush (output_stream);
+  gnulib::fflush (output_stream);
 
   return octave_fgetl (input_stream, eof);
 }
@@ -831,7 +832,12 @@ command_editor::instance_ok (void)
   bool retval = true;
 
   if (! instance)
-    make_command_editor ();
+    {
+      make_command_editor ();
+
+      if (instance)
+        singleton_cleanup_list::add (cleanup_instance);
+    }
 
   if (! instance)
     {
@@ -1226,6 +1232,12 @@ command_editor::remove_event_hook (event_hook_fcn f)
     }
 }
 
+void
+command_editor::run_event_hooks (void)
+{
+  event_handler ();
+}
+
 void
 command_editor::read_init_file (const std::string& file_arg)
 {
