@@ -1,5 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2004, 2005, 2006,
-##               2007 John W. Eaton
+## Copyright (C) 1995-2012 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -19,11 +18,11 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} loadaudio (@var{name}, @var{ext}, @var{bps})
-## Loads audio data from the file @file{@var{name}.@var{ext}} into the
+## Load audio data from the file @file{@var{name}.@var{ext}} into the
 ## vector @var{x}.
 ##
 ## The extension @var{ext} determines how the data in the audio file is
-## interpreted;  the extensions @file{lin} (default) and @file{raw}
+## interpreted; the extensions @file{lin} (default) and @file{raw}
 ## correspond to linear, the extensions @file{au}, @file{mu}, or @file{snd}
 ## to mu-law encoding.
 ##
@@ -37,7 +36,7 @@
 ## Created: 10 April 1994
 ## Adapted-By: jwe
 
-function X = loadaudio (name, ext, bit)
+function X = loadaudio (name, ext, bps)
 
   if (nargin == 0 || nargin > 3)
     print_usage ();
@@ -48,33 +47,33 @@ function X = loadaudio (name, ext, bit)
   endif
 
   if (nargin < 3)
-    bit = 8;
-  elseif (bit != 8 && bit != 16)
-    error ("loadaudio: bit must be either 8 or 16");
+    bps = 8;
+  elseif (bps != 8 && bps != 16)
+    error ("loadaudio: BPS must be either 8 or 16");
   endif
 
   name = [name, ".", ext];
   num = fopen (name, "rb");
 
   if (strcmp (ext, "lin") || strcmp (ext, "raw") || strcmp (ext, "pcm"))
-    if (bit == 8)
+    if (bps == 8)
       [Y, c] = fread (num, inf, "uchar");
       X = Y - 127;
     else
       [X, c] = fread (num, inf, "short");
     endif
   elseif (strcmp (ext, "mu") || strcmp (ext, "au")
-	  || strcmp (ext, "snd") || strcmp(ext, "ul"))
+          || strcmp (ext, "snd") || strcmp(ext, "ul"))
     [Y, c] = fread (num, inf, "uchar");
     ## remove file header
     m = find (Y(1:64) == 0, 1, "last");
     if (! isempty (m))
       Y(1:m) = [];
     endif
-    X = mu2lin (Y, bit);
+    X = mu2lin (Y, bps);
   else
     fclose (num);
-    error ("loadaudio does not support given extension");
+    error ("loadaudio: unsupported extension");
   endif
 
   fclose (num);
