@@ -1,5 +1,4 @@
-## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2005, 2006, 2007
-##               John W. Eaton
+## Copyright (C) 1994-2012 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,30 +17,39 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} gray (@var{n})
-## Return a gray colormap with @var{n} entries corresponding to values from
-## 0 to @var{n}-1.  The argument @var{n} should be a scalar.  If it is
-## omitted, the length of the current colormap or 64 is assumed.
+## @deftypefn  {Function File} {@var{map} =} gray ()
+## @deftypefnx {Function File} {@var{map} =} gray (@var{n})
+## Create gray colormap.  This colormap varies from black to white with
+## shades of gray.
+## The argument @var{n} must be a scalar.
+## If unspecified, the length of the current colormap, or 64, is used.
 ## @end deftypefn
 
 ## Author: Tony Richardson <arichard@stark.cc.oh.us>
 ## Created: July 1994
 ## Adapted-By: jwe
 
-function map = gray (number)
+function map = gray (n)
 
   if (nargin == 0)
-    number = rows (colormap);
+    n = rows (colormap);
   elseif (nargin == 1)
-    if (! isscalar (number))
+    if (! isscalar (n))
       error ("gray: argument must be a scalar");
     endif
   else
     print_usage ();
   endif
 
-  gr = [0:(number-1)]';
+  gr = [0:(n-1)]';
 
-  map = [ gr, gr, gr ] / (number - 1);
+  map = [ gr, gr, gr ] / (n - 1);
 
 endfunction
+
+%!demo
+%! ## Show the 'gray' colormap as an image
+%! image (1:64, linspace (0, 1, 64), repmat (1:64, 64, 1)')
+%! axis ([1, 64, 0, 1], "ticy", "xy")
+%! colormap (gray (64))
+
