@@ -1,4 +1,5 @@
-## Copyright (C) 2000, 2006, 2007 Etienne Grossmann
+## Copyright (C) 2000-2012 Etienne Grossmann
+## Copyright (C) 2009 VZLU Prague
 ##
 ## This file is part of Octave.
 ##
@@ -18,22 +19,25 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {[@var{k1}, @dots{}, @var{v1}] =} setfield (@var{s}, @var{k1}, @var{v1}, @dots{})
-## Set field members in a structure.
+## Set a field member in a (nested) structure array.  For example:
 ##
 ## @example
 ## @group
-## oo(1,1).f0= 1;
-## oo = setfield(oo,@{1,2@},'fd',@{3@},'b', 6);
+## oo(1,1).f0 = 1;
+## oo = setfield (oo, @{1,2@}, "fd", @{3@}, "b", 6);
 ## oo(1,2).fd(3).b == 6
-## @result{} ans = 1
+##      @result{} ans = 1
 ## @end group
 ## @end example
 ##
-## Note that this function could be written
+## Note that the same result as in the above example could be achieved by:
 ##
 ## @example
-##          i1= @{1,2@}; i2= 'fd'; i3= @{3@}; i4= 'b';
-##          oo( i1@{:@} ).( i2 )( i3@{:@} ).( i4 ) == 6;
+## @group
+## i1 = @{1,2@}; i2 = "fd"; i3 = @{3@}; i4 = "b";
+## oo(i1@{:@}).(i2)(i3@{:@}).(i4) == 6
+##      @result{} ans = 1
+## @end group
 ## @end example
 ## @seealso{getfield, rmfield, isfield, isstruct, fieldnames, struct}
 ## @end deftypefn
@@ -41,30 +45,27 @@
 ## Author:  Etienne Grossmann <etienne@cs.uky.edu>
 
 function obj = setfield (obj, varargin)
-   field = "obj";
-   for i = 1:nargin-2
-     v = varargin{i};
-     if (iscell (v))
-       sep = "(";
-       for j = 1:length (v)
-	 field = sprintf ("%s%s%s", field, sep, num2str (v{j}));
-         sep = ",";
-       endfor
-       field = sprintf ("%s)", field);
-     else
-       field = sprintf ("%s.%s", field, v);
-     endif
-   endfor
-   val = varargin{nargin-1};
-   eval (sprintf ("%s=val;", field));
+  if (nargin < 3)
+    print_usage ();
+  endif
+  subs = varargin(1:end-1);
+  rhs = varargin{end};
+  flds = cellfun ("isclass", subs, "char");
+  idxs = cellfun ("isclass", subs, "cell");
+  if (all (flds | idxs))
+    typs = merge (flds, {"."}, {"()"});
+    obj = subsasgn (obj, struct ("type", typs, "subs", subs), rhs);
+  else
+    error ("setfield: invalid index");
+  endif
 endfunction
 
 %!test
 %! x.a = "hello";
 %! x = setfield(x,"b","world");
-%! y = struct('a','hello','b','world');
+%! y = struct("a","hello","b","world");
 %! assert(x,y);
 %!test
 %! oo(1,1).f0= 1;
-%! oo = setfield(oo,{1,2},'fd',{3},'b', 6);
+%! oo = setfield(oo,{1,2},"fd",{3},"b", 6);
 %! assert (oo(1,2).fd(3).b, 6)
