@@ -1,5 +1,5 @@
-## Copyright (C) 2005, 2006, 2007 Sren Hauberg
-## 
+## Copyright (C) 2005-2012 Sren Hauberg
+##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
@@ -17,30 +17,27 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} untar (@var{tarfile}, @var{dir})
+## @deftypefn  {Function File} {} untar (@var{tarfile})
+## @deftypefnx {Function File} {} untar (@var{tarfile}, @var{dir})
 ## Unpack the TAR archive @var{tarfile} to the directory @var{dir}.
 ## If @var{dir} is not specified, it defaults to the current directory.
-## @seealso{unpack, bzip2, bunzip2, tar, gzip, gunzip, zip, unzip}
+## @seealso{tar, unpack, bunzip2, gunzip, unzip}
 ## @end deftypefn
 
 ## Author: Sren Hauberg <hauberg@gmail.com>
 ## Adapted-By: jwe, Bill Denney
 
-function varargout = untar (files, outputdir)
+function varargout = untar (tarfile, dir = ".")
 
-  if (! (nargin == 1 || nargin == 2))
+  if (nargin != 1 && nargin != 2)
     print_usage ();
   endif
 
-  if (nargin == 1)
-    outputdir = ".";
-  endif
-
   if (nargout > 0)
     varargout = cell (1, nargout);
-    [varargout{:}] = unpack (files, outputdir, mfilename ());
+    [varargout{:}] = unpack (tarfile, dir, mfilename ());
   else
-    unpack (files, outputdir, mfilename ());
+    unpack (tarfile, dir, mfilename ());
   endif
 
 endfunction
