@@ -1,5 +1,4 @@
-## Copyright (C) 1994, 1995, 1996, 1997, 2000, 2004, 2005, 2006, 2007
-##               John W. Eaton
+## Copyright (C) 1994-2012 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,7 +17,7 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} shg
+## @deftypefn {Command} {} shg
 ## Show the graph window.  Currently, this is the same as executing
 ## @code{drawnow}.
 ## @seealso{drawnow, figure}
