@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2006,
-              2007 John W. Eaton
+Copyright (C) 1996-2012 John W. Eaton
 
 This file is part of Octave.
 
@@ -24,7 +23,7 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_tree_assign_h)
 #define octave_tree_assign_h 1
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 class tree_argument_list;
@@ -36,6 +35,7 @@ class octave_lvalue;
 
 #include "ov.h"
 #include "pt-exp.h"
+#include "symtab.h"
 
 // Simple assignment expressions.
 
@@ -45,13 +45,13 @@ tree_simple_assignment : public tree_expression
 public:
 
   tree_simple_assignment (bool plhs = false, int l = -1, int c = -1,
-			  octave_value::assign_op t = octave_value::op_asn_eq)
-    : tree_expression (l, c), lhs (0), rhs (0), preserve (plhs), etype (t),
-      first_execution (true) { }
+                          octave_value::assign_op t = octave_value::op_asn_eq)
+    : tree_expression (l, c), lhs (0), rhs (0), preserve (plhs), ans_ass (),
+      etype (t), first_execution (true) { }
 
   tree_simple_assignment (tree_expression *le, tree_expression *re,
-			  bool plhs = false, int l = -1, int c = -1,
-			  octave_value::assign_op t = octave_value::op_asn_eq);
+                          bool plhs = false, int l = -1, int c = -1,
+                          octave_value::assign_op t = octave_value::op_asn_eq);
 
   ~tree_simple_assignment (void);
 
@@ -59,21 +59,20 @@ public:
 
   bool rvalue_ok (void) const { return true; }
 
-  octave_value rvalue (void);
+  octave_value rvalue1 (int nargout = 1);
 
   octave_value_list rvalue (int nargout);
 
   bool is_assignment_expression (void) const { return true; }
 
-  void eval_error (void);
-
   std::string oper (void) const;
 
   tree_expression *left_hand_side (void) { return lhs; }
 
   tree_expression *right_hand_side (void) { return rhs; }
 
-  tree_expression *dup (symbol_table *sym_tab);
+  tree_expression *dup (symbol_table::scope_id scope,
+                        symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -82,7 +81,7 @@ public:
 private:
 
   void do_assign (octave_lvalue& ult, const octave_value_list& args,
-		  const octave_value& rhs_val);
+                  const octave_value& rhs_val);
 
   void do_assign (octave_lvalue& ult, const octave_value& rhs_val);
 
@@ -118,14 +117,12 @@ tree_multi_assignment : public tree_expression
 {
 public:
 
-  tree_multi_assignment (bool plhs = false, int l = -1, int c = -1,
-			 octave_value::assign_op t = octave_value::op_asn_eq)
-    : tree_expression (l, c), lhs (0), rhs (0), preserve (plhs), etype(t),
+  tree_multi_assignment (bool plhs = false, int l = -1, int c = -1)
+    : tree_expression (l, c), lhs (0), rhs (0), preserve (plhs),
       first_execution (true) { }
 
   tree_multi_assignment (tree_argument_list *lst, tree_expression *r,
-			 bool plhs = false, int l = -1, int c = -1,
-			 octave_value::assign_op t = octave_value::op_asn_eq);
+                         bool plhs = false, int l = -1, int c = -1);
 
   ~tree_multi_assignment (void);
 
@@ -135,23 +132,22 @@ public:
 
   bool rvalue_ok (void) const { return true; }
 
-  octave_value rvalue (void);
+  octave_value rvalue1 (int nargout = 1);
 
   octave_value_list rvalue (int nargout);
 
-  void eval_error (void);
-
   std::string oper (void) const;
 
   tree_argument_list *left_hand_side (void) { return lhs; }
 
   tree_expression *right_hand_side (void) { return rhs; }
 
-  tree_expression *dup (symbol_table *sym_tab);
+  tree_expression *dup (symbol_table::scope_id scope,
+                        symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
-  
-  octave_value::assign_op op_type (void) const { return etype; }
+
+  octave_value::assign_op op_type (void) const { return octave_value::op_asn_eq; }
 
 private:
 
@@ -164,9 +160,6 @@ private:
   // True if we should not delete the lhs.
   bool preserve;
 
-  // The type of the expression.
-  octave_value::assign_op etype;
-
   // true only on first rvalue() call.
   bool first_execution;
 
@@ -178,9 +171,3 @@ private:
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
