@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1999-2012 John W. Eaton
 
 This file is part of Octave.
 
@@ -41,12 +40,10 @@ along with Octave; see the file COPYING.  If not, see
 #include "variables.h"
 
 octave_value
-tree_cell::rvalue (void)
+tree_cell::rvalue1 (int)
 {
   octave_value retval;
 
-  MAYBE_DO_BREAKPOINT;
-
   octave_idx_type nr = length ();
   octave_idx_type nc = -1;
 
@@ -59,26 +56,29 @@ tree_cell::rvalue (void)
       tree_argument_list *elt = *p;
 
       octave_value_list row = elt->convert_to_const_vector ();
-      
-      if (nc < 0)
-	{
-	  nc = row.length ();
 
-	  val = Cell (nr, nc);
-	}
+      if (nr == 1)
+        // Optimize the single row case.
+        val = row.cell_value ();
+      else if (nc < 0)
+        {
+          nc = row.length ();
+
+          val = Cell (nr, nc);
+        }
       else
-	{
-	  octave_idx_type this_nc = row.length ();
+        {
+          octave_idx_type this_nc = row.length ();
 
-	  if (nc != this_nc)
-	    {
-	      ::error ("number of columns must match");
-	      return retval;
-	    }
-	}
+          if (nc != this_nc)
+            {
+              ::error ("number of columns must match");
+              return retval;
+            }
+        }
 
       for (octave_idx_type j = 0; j < nc; j++)
-	val(i,j) = row(j);
+        val(i,j) = row(j);
 
       i++;
     }
@@ -96,21 +96,22 @@ tree_cell::rvalue (int nargout)
   if (nargout > 1)
     error ("invalid number of output arguments for cell array");
   else
-    retval = rvalue ();
+    retval = rvalue1 (nargout);
 
   return retval;
 }
 
 tree_expression *
-tree_cell::dup (symbol_table *sym_tab)
+tree_cell::dup (symbol_table::scope_id scope,
+                symbol_table::context_id context) const
 {
   tree_cell *new_cell = new tree_cell (0, line (), column ());
 
-  for (iterator p = begin (); p != end (); p++)
+  for (const_iterator p = begin (); p != end (); p++)
     {
-      tree_argument_list *elt = *p;
+      const tree_argument_list *elt = *p;
 
-      new_cell->append (elt ? elt->dup (sym_tab) : 0);
+      new_cell->append (elt ? elt->dup (scope, context) : 0);
     }
 
   new_cell->copy_base (*this);
@@ -123,9 +124,3 @@ tree_cell::accept (tree_walker& tw)
 {
   tw.visit_cell (*this);
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
