@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1999-2012 John W. Eaton
 
 This file is part of Octave.
 
@@ -24,7 +23,7 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_tree_cell_h)
 #define octave_tree_cell_h 1
 
-#include <iostream>
+#include <iosfwd>
 
 class octave_value;
 class octave_value_list;
@@ -33,6 +32,7 @@ class tree_argument_list;
 class tree_walker;
 
 #include "pt-mat.h"
+#include "symtab.h"
 
 // General cells.
 
@@ -48,11 +48,12 @@ public:
 
   bool rvalue_ok (void) const { return true; }
 
-  octave_value rvalue (void);
+  octave_value rvalue1 (int nargout = 1);
 
   octave_value_list rvalue (int);
 
-  tree_expression *dup (symbol_table *sym_tab);
+  tree_expression *dup (symbol_table::scope_id scope,
+                        symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -66,9 +67,3 @@ private:
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
